package jettyTiddlywiki.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jettyTiddlywiki.entity.Tidder;
import jettyTiddlywiki.factory.B32F;
import jettyTiddlywiki.io.DiffIO;
import jettyTiddlywiki.io.FileIO;
import jettyTiddlywiki.io.TidderIO;

public class Initializer extends HttpServlet
{
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
    	initialize();
		try {
			response.sendRedirect("/wiki/index");
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    private void initialize(){
    	createDB();
    	createMainMenu();
    	createPageTemplate();
    	createStyleSheetLayout();
    }
    
    private void createMainMenu(){
    	Tidder ct = TidderIO.readTidder(B32F.get().encodeToString("MainMenu".getBytes()));
    	if(ct == null){
	    	Tidder t = new Tidder("MainMenu", MAINMENUSTR);
	    	TidderIO.writeTidder(t);
    	}
    }
    private void createPageTemplate(){
    	Tidder pt = TidderIO.readTidder(B32F.get().encodeToString("PageTemplate".getBytes()));
    	if(pt == null){
	       	final StringBuilder msb = new StringBuilder();
	    	try
	        {
	    		ServletContext context = getServletContext();
            	final URL u = context.getResource("/WEB-INF/tmpl.txt");
	            final BufferedReader in = new BufferedReader(new InputStreamReader(u.openStream(), "UTF-8"));
	            String inputLine;
	            while ((inputLine = in.readLine()) != null)
	            {
	                msb.append(inputLine);
	                msb.append("\n");
	            }
	            in.close();
	        }
	        catch (final IOException e1)
	        {
	            e1.printStackTrace();
	        }
	        Tidder t = new Tidder("PageTemplate", msb.toString().trim());
	    	TidderIO.writeTidder(t);
    	}
    }
    private void createStyleSheetLayout(){
    	Tidder ct = TidderIO.readTidder(B32F.get().encodeToString("StyleSheetLayout".getBytes()));
    	if(ct == null){
	       	final StringBuilder msb = new StringBuilder();
	    	try
	        {
	    		ServletContext context = getServletContext();
            	final URL u = context.getResource("/WEB-INF/wiki.css");
	            final BufferedReader in = new BufferedReader(new InputStreamReader(u.openStream(), "UTF-8"));
	            String inputLine;
	            while ((inputLine = in.readLine()) != null)
	            {
	                msb.append(inputLine);
	                msb.append("\n");
	            }
	            in.close();
	        }
	        catch (final IOException e1)
	        {
	            e1.printStackTrace();
	        }
	        Tidder t = new Tidder("StyleSheetLayout", msb.toString().trim());
	    	TidderIO.writeTidder(t);
    	}
    }
    private void createDB(){
    	TidderIO.init();
    	DiffIO.init();
    	FileIO.init();
    }

    
    private static String MAINMENUSTR = "[[Home|/]]\n[[PageTemplate]]\n[[MainMenu]]\n[[StyleSheetLayout]]";
}
