package jettyTiddlywiki.servlet;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.EncoderException;

import difflib.Delta;
import difflib.DiffUtils;
import difflib.Patch;

import jettyTiddlywiki.entity.Tidder;
import jettyTiddlywiki.entity.Diff;
import jettyTiddlywiki.factory.B32F;
import jettyTiddlywiki.factory.URCF;
import jettyTiddlywiki.io.DiffIO;
import jettyTiddlywiki.io.TidderIO;
import jettyTiddlywiki.response.AbnormalPrinter;
import jettyTiddlywiki.response.AbnormalPrinter.ResCode;
import jettyTiddlywiki.toolbox.Miscellaneous;
import jettyTiddlywiki.url.URLHandler;

public class Wiki extends HttpServlet
{

    @Override
    public void doGet(final HttpServletRequest req, final HttpServletResponse res)
    {
    	if(!URLHandler.INSTANCE.handle(req, res, URLHandler.PC.Wiki)) return;
    	wiki(req, res, new File(req.getPathInfo()).getName());
    }
    
    private void wiki(final HttpServletRequest req, final HttpServletResponse res, final String s)
    {
    	//null ok
    	Tidder _or = TidderIO.readTidder(B32F.get().encodeToString(s.getBytes()));
        
        Tidder _pt = TidderIO.readTidder(B32F.get().encodeToString("PageTemplate".getBytes()));
        Tidder _mm = TidderIO.readTidder(B32F.get().encodeToString("MainMenu".getBytes()));
        
        res.setContentType("text/html");
        res.setCharacterEncoding("UTF-8");
        try {
			final PrintWriter out = res.getWriter();
			out.write(
					_pt.getRaw()
					.replace("{{{MainMenu}}}",_mm.getHtml())
					.replace("{{{wiki.title}}}",s)
					.replace("{{{wiki.title.encoded}}}",URCF.encode(s))//%25 + /coms/%E6%97%A5%E6%9C%AC%E8%AA%9E+%E3%82%B9%E3%83%9A%EF%BC%BE%E3%81%99
					.replace("{{{modified}}}",(_or==null)?"":new SimpleDateFormat("dd MMMMM yyyy").format(Miscellaneous.INSTANCE.defsdf.parse(_or.getTimeStamp())))
					.replace("{{{wiki.html}}}",(_or==null)?"":_or.getHtml().replace("{{{", "&#123;&#123;&#123;").replace("}}}", "&#125;&#125;&#125;"))
					.replace("{{{wiki.raw}}}",(_or==null)?"":_or.getRaw().replace("<", "&lt;").replace(">", "&gt;"))
					);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParseException e) {
			e.printStackTrace();
		}
    }

    @Override
    public void doPost(final HttpServletRequest req, final HttpServletResponse res)
    {
    	if(!URLHandler.INSTANCE.handle(req, res, URLHandler.PC.Wiki)) return;
    	String name = new File(req.getPathInfo()).getName();

        final String raw = req.getParameter("raw").replaceAll("\r\n?", "\n");
       
        Tidder _or = TidderIO.readTidder(B32F.get().encodeToString(name.getBytes()));
        Diff diff_or = DiffIO.readDiff(B32F.get().encodeToString(name.getBytes()));
        
        final String _diff = (diff_or != null) ? diff_or.getHtml() : "";
        final String _raw = (_or != null) ? _or.getRaw() : "";
        final List<String> original = Arrays.asList(_raw.split("\n"));
        final List<String> revised = Arrays.asList(raw.split("\n"));

    	StringBuilder sb = new StringBuilder();
        sb.append("<div class='date'>" + new SimpleDateFormat("yyyy.MM.dd kk:mm:ss z EEEEE").format(new Date()) + "</div>\n");
    	Patch patch = DiffUtils.diff(original, revised);
    	List<Delta> deltas = patch.getDeltas();
    	int last = 0;
    	for (Delta delta : deltas) {
    		if(last + 1 < delta.getOriginal().getPosition()){} // not continuous
		    List<?> or = delta.getOriginal().getLines();
    		List<?> re = delta.getRevised().getLines();
    		String orText = Miscellaneous.INSTANCE.join(or).replace("<", "&lt;").replace(">", "&gt;");
    		String reText = Miscellaneous.INSTANCE.join(re).replace("<", "&lt;").replace(">", "&gt;");
    		if(orText.trim().isEmpty()){
    			sb.append("<pre class='insert'>" + reText + "</pre>\n");
    		}else if(reText.trim().isEmpty()){
    			sb.append("<pre class='del'>" + orText + "</pre>\n");
    		}else{
                sb.append("<div class='mod'>\n");
                sb.append("\t<pre class='insert'>" + reText + "</pre>\n");
                sb.append("\t<pre class='del'>" + orText + "</pre>\n");
                sb.append("</div>\n");
    		}
			last = delta.getOriginal().last();
		}
		if(last + 1 < original.size()){}//last is not delta
		sb.append(_diff);
		
        // override
        final Tidder nt = new Tidder(name,raw);
        final Diff nd = new Diff(name,sb.toString());
        TidderIO.writeTidder(nt);
        DiffIO.writeDiff(nd);
    }
}
